﻿using System.ComponentModel;
using System.Threading.Tasks;
using System.IO;
using Plugin.Media.Abstractions;
using Syncfusion.SfImageEditor.XForms;
using Xamarin.Forms;
using Xamarin.Essentials;
using System;
using System.Linq;
using Plugin.Media;

namespace IESample
{
    public partial class ImageEditorGettingStarted : ContentPage
    {


        private AddPhotoViewModel Modello { get; set; }

        private readonly MediaFile image;
        public ImageEditorGettingStarted(MediaFile photo)
        {
            InitializeComponent();

            Modello = new AddPhotoViewModel();
            BindingContext = Modello;

            image = photo;
            LoadImageEditor(photo.GetStream());
        }

        private void LoadImageEditor(Stream stream)
        {
            Modello.Immagine = ImageSource.FromStream(() => stream);
            editor.Source = Modello.Immagine;
        }

        private void editor_ImageLoaded(object sender, ImageLoadedEventArgs args)
        {
            // I need the toolbar updates and the control selects automatically the Path button and its subitems become active
            // This way the user can start to draw on the image as it is loaded
            // Is it possible?
            Syncfusion.SfImageEditor.XForms.ToolbarItem toolbarItem = editor.ToolbarSettings.ToolbarItems.Where(w => w.Name == "Path").FirstOrDefault();
            ToolbarSettings_ToolbarItemSelected(null, new ToolbarItemSelectedEventArgs(toolbarItem));
        }

        private void ToolbarSettings_ToolbarItemSelected(object sender, ToolbarItemSelectedEventArgs e)
        {
            string text = e.ToolbarItem.Name;

            if (text.Equals("Reset"))
            {
                Modello.Immagine = null;
                editor.Source = Modello.Immagine;
            }
        }

        protected override void OnAppearing()
        {
            base.OnAppearing();

            MessagingCenter.Subscribe<AddPhotoViewModel>(this, "DisplayAlertMessage", async (sender) =>
            {
                if (await DisplayAlert("Title", "Add another image?", "OK", "KO"))
                {
                    Syncfusion.SfImageEditor.XForms.ToolbarItem toolbarItem = editor.ToolbarSettings.ToolbarItems.Where(w => w.Name == "Reset").FirstOrDefault();
                    ToolbarSettings_ToolbarItemSelected(null, new ToolbarItemSelectedEventArgs(toolbarItem));
                    Modello.SetNomeFile();

                    MediaFile file = await CrossMedia.Current.TakePhotoAsync(
                        new StoreCameraMediaOptions
                        {
                            MaxWidthHeight = 1200,
                            SaveToAlbum = false,
                            PhotoSize = PhotoSize.MaxWidthHeight,
                            CompressionQuality = 100,
                            AllowCropping = false
                        }
                    );

                    if (file != null)
                        LoadImageEditor(file.GetStream());
                }
                else
                    await Navigation.PopAsync();
            });
        }

        protected override void OnDisappearing()
        {
            MessagingCenter.Unsubscribe<AddPhotoViewModel>(this, "DisplayAlertMessage");
            base.OnDisappearing();
        }
    }

    public class AddPhotoViewModel
    {
        private ImageSource immagine;

        public AddPhotoViewModel()
        {
            SaveAttachCommand = new Command<Stream>(async (s) => await ExecuteSaveAttachCommand(s));
            SetNomeFile();
        }

        public ImageSource Immagine
        {
            get { return immagine; }
            set
            {
                immagine = value;
                OnPropertyChanged(nameof(Immagine));
            }
        }

        private string pathImageFile { get; set; }
        public string PathImageFile
        {
            get { return pathImageFile; }
            set
            {
                pathImageFile = value;
                OnPropertyChanged(nameof(PathImageFile));
            }
        }

        public Command<Stream> SaveAttachCommand { get; set; }


        #region PropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
        #endregion

        public void SetNomeFile()
        {
            string folderPath = Path.Combine(FileSystem.AppDataDirectory, "Attach");

            if (!Directory.Exists(folderPath))
                Directory.CreateDirectory(folderPath);

            DateTime dateTime = DateTime.Now;
            string nomeFile = $"{Path.GetRandomFileName()}_{dateTime.ToString("yyyyMMddHHmmss")}.png";
            PathImageFile = Path.Combine(folderPath, nomeFile);
        }

        private async Task ExecuteSaveAttachCommand(Stream stream)
        {
            if (!string.IsNullOrEmpty(PathImageFile))
            {
                using (MemoryStream memoryStreamHandler = new MemoryStream())
                {
                    stream.CopyTo(memoryStreamHandler);
                    byte[] fileContent = memoryStreamHandler.ToArray();

                    File.WriteAllBytes(PathImageFile, fileContent);

                    MessagingCenter.Send(this, "DisplayAlertMessage");
                }
            }
        }
    }
}